@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2011 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2016 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,7 +18,7 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "SDL_config.h"
+#include "../../SDL_internal.h"
 
 #if SDL_AUDIO_DRIVER_ALSA
 
@@ -155,7 +155,7 @@ static void
 UnloadALSALibrary(void)
 {
     if (alsa_handle != NULL) {
-		SDL_UnloadObject(alsa_handle);
+        SDL_UnloadObject(alsa_handle);
         alsa_handle = NULL;
     }
 }
@@ -241,25 +241,25 @@ ALSA_WaitDevice(_THIS)
         tmp = ptr[3]; ptr[3] = ptr[5]; ptr[5] = tmp; \
     }
 
-static __inline__ void
+static SDL_INLINE void
 swizzle_alsa_channels_6_64bit(_THIS)
 {
     SWIZ6(Uint64);
 }
 
-static __inline__ void
+static SDL_INLINE void
 swizzle_alsa_channels_6_32bit(_THIS)
 {
     SWIZ6(Uint32);
 }
 
-static __inline__ void
+static SDL_INLINE void
 swizzle_alsa_channels_6_16bit(_THIS)
 {
     SWIZ6(Uint16);
 }
 
-static __inline__ void
+static SDL_INLINE void
 swizzle_alsa_channels_6_8bit(_THIS)
 {
     SWIZ6(Uint8);
@@ -272,7 +272,7 @@ swizzle_alsa_channels_6_8bit(_THIS)
  * Called right before feeding this->hidden->mixbuf to the hardware. Swizzle
  *  channels from Windows/Mac order to the format alsalib will want.
  */
-static __inline__ void
+static SDL_INLINE void
 swizzle_alsa_channels(_THIS)
 {
     if (this->spec.channels == 6) {
@@ -304,7 +304,7 @@ ALSA_PlayDevice(_THIS)
 
     while ( frames_left > 0 && this->enabled ) {
         /* !!! FIXME: This works, but needs more testing before going live */
-        /*ALSA_snd_pcm_wait(this->hidden->pcm_handle, -1);*/
+        /* ALSA_snd_pcm_wait(this->hidden->pcm_handle, -1); */
         status = ALSA_snd_pcm_writei(this->hidden->pcm_handle,
                                      sample_buf, frames_left);
 
@@ -320,7 +320,7 @@ ALSA_PlayDevice(_THIS)
                 /* Hmm, not much we can do - abort */
                 fprintf(stderr, "ALSA write failed (unrecoverable): %s\n",
                         ALSA_snd_strerror(status));
-                this->enabled = 0;
+                SDL_OpenedAudioDeviceDisconnected(this);
                 return;
             }
             continue;
@@ -340,10 +340,8 @@ static void
 ALSA_CloseDevice(_THIS)
 {
     if (this->hidden != NULL) {
-        if (this->hidden->mixbuf != NULL) {
-            SDL_FreeAudioMem(this->hidden->mixbuf);
-            this->hidden->mixbuf = NULL;
-        }
+        SDL_FreeAudioMem(this->hidden->mixbuf);
+        this->hidden->mixbuf = NULL;
         if (this->hidden->pcm_handle) {
             ALSA_snd_pcm_drain(this->hidden->pcm_handle);
             ALSA_snd_pcm_close(this->hidden->pcm_handle);
@@ -467,7 +465,7 @@ ALSA_set_buffer_size(_THIS, snd_pcm_hw_params_t *params, int override)
 }
 
 static int
-ALSA_OpenDevice(_THIS, const char *devname, int iscapture)
+ALSA_OpenDevice(_THIS, void *handle, const char *devname, int iscapture)
 {
     int status = 0;
     snd_pcm_t *pcm_handle = NULL;
@@ -482,8 +480,7 @@ ALSA_OpenDevice(_THIS, const char *devname, int iscapture)
     this->hidden = (struct SDL_PrivateAudioData *)
         SDL_malloc((sizeof *this->hidden));
     if (this->hidden == NULL) {
-        SDL_OutOfMemory();
-        return 0;
+        return SDL_OutOfMemory();
     }
     SDL_memset(this->hidden, 0, (sizeof *this->hidden));
 
@@ -495,9 +492,8 @@ ALSA_OpenDevice(_THIS, const char *devname, int iscapture)
 
     if (status < 0) {
         ALSA_CloseDevice(this);
-        SDL_SetError("ALSA: Couldn't open audio device: %s",
-                     ALSA_snd_strerror(status));
-        return 0;
+        return SDL_SetError("ALSA: Couldn't open audio device: %s",
+                            ALSA_snd_strerror(status));
     }
 
     this->hidden->pcm_handle = pcm_handle;
@@ -507,9 +503,8 @@ ALSA_OpenDevice(_THIS, const char *devname, int iscapture)
     status = ALSA_snd_pcm_hw_params_any(pcm_handle, hwparams);
     if (status < 0) {
         ALSA_CloseDevice(this);
-        SDL_SetError("ALSA: Couldn't get hardware config: %s",
-                     ALSA_snd_strerror(status));
-        return 0;
+        return SDL_SetError("ALSA: Couldn't get hardware config: %s",
+                            ALSA_snd_strerror(status));
     }
 
     /* SDL only uses interleaved sample output */
@@ -517,9 +512,8 @@ ALSA_OpenDevice(_THIS, const char *devname, int iscapture)
                                                SND_PCM_ACCESS_RW_INTERLEAVED);
     if (status < 0) {
         ALSA_CloseDevice(this);
-        SDL_SetError("ALSA: Couldn't set interleaved access: %s",
+        return SDL_SetError("ALSA: Couldn't set interleaved access: %s",
                      ALSA_snd_strerror(status));
-        return 0;
     }
 
     /* Try for a closest match on audio format */
@@ -572,8 +566,7 @@ ALSA_OpenDevice(_THIS, const char *devname, int iscapture)
     }
     if (status < 0) {
         ALSA_CloseDevice(this);
-        SDL_SetError("ALSA: Couldn't find any hardware audio formats");
-        return 0;
+        return SDL_SetError("ALSA: Couldn't find any hardware audio formats");
     }
     this->spec.format = test_format;
 
@@ -585,8 +578,7 @@ ALSA_OpenDevice(_THIS, const char *devname, int iscapture)
         status = ALSA_snd_pcm_hw_params_get_channels(hwparams, &channels);
         if (status < 0) {
             ALSA_CloseDevice(this);
-            SDL_SetError("ALSA: Couldn't set audio channels");
-            return 0;
+            return SDL_SetError("ALSA: Couldn't set audio channels");
         }
         this->spec.channels = channels;
     }
@@ -597,9 +589,8 @@ ALSA_OpenDevice(_THIS, const char *devname, int iscapture)
                                                   &rate, NULL);
     if (status < 0) {
         ALSA_CloseDevice(this);
-        SDL_SetError("ALSA: Couldn't set audio frequency: %s",
-                     ALSA_snd_strerror(status));
-        return 0;
+        return SDL_SetError("ALSA: Couldn't set audio frequency: %s",
+                            ALSA_snd_strerror(status));
     }
     this->spec.freq = rate;
 
@@ -609,8 +600,7 @@ ALSA_OpenDevice(_THIS, const char *devname, int iscapture)
         /* Failed to set desired buffer size, do the best you can... */
         if ( ALSA_set_period_size(this, hwparams, 1) < 0 ) {
             ALSA_CloseDevice(this);
-            SDL_SetError("Couldn't set hardware audio parameters: %s", ALSA_snd_strerror(status));
-            return(-1);
+            return SDL_SetError("Couldn't set hardware audio parameters: %s", ALSA_snd_strerror(status));
         }
     }
     /* Set the software parameters */
@@ -618,31 +608,27 @@ ALSA_OpenDevice(_THIS, const char *devname, int iscapture)
     status = ALSA_snd_pcm_sw_params_current(pcm_handle, swparams);
     if (status < 0) {
         ALSA_CloseDevice(this);
-        SDL_SetError("ALSA: Couldn't get software config: %s",
-                     ALSA_snd_strerror(status));
-        return 0;
+        return SDL_SetError("ALSA: Couldn't get software config: %s",
+                            ALSA_snd_strerror(status));
     }
     status = ALSA_snd_pcm_sw_params_set_avail_min(pcm_handle, swparams, this->spec.samples);
     if (status < 0) {
         ALSA_CloseDevice(this);
-        SDL_SetError("Couldn't set minimum available samples: %s",
-                     ALSA_snd_strerror(status));
-        return 0;
+        return SDL_SetError("Couldn't set minimum available samples: %s",
+                            ALSA_snd_strerror(status));
     }
     status =
         ALSA_snd_pcm_sw_params_set_start_threshold(pcm_handle, swparams, 1);
     if (status < 0) {
         ALSA_CloseDevice(this);
-        SDL_SetError("ALSA: Couldn't set start threshold: %s",
-                     ALSA_snd_strerror(status));
-        return 0;
+        return SDL_SetError("ALSA: Couldn't set start threshold: %s",
+                            ALSA_snd_strerror(status));
     }
     status = ALSA_snd_pcm_sw_params(pcm_handle, swparams);
     if (status < 0) {
         ALSA_CloseDevice(this);
-        SDL_SetError("Couldn't set software audio parameters: %s",
-                     ALSA_snd_strerror(status));
-        return 0;
+        return SDL_SetError("Couldn't set software audio parameters: %s",
+                            ALSA_snd_strerror(status));
     }
 
     /* Calculate the final parameters for this audio specification */
@@ -653,16 +639,15 @@ ALSA_OpenDevice(_THIS, const char *devname, int iscapture)
     this->hidden->mixbuf = (Uint8 *) SDL_AllocAudioMem(this->hidden->mixlen);
     if (this->hidden->mixbuf == NULL) {
         ALSA_CloseDevice(this);
-        SDL_OutOfMemory();
-        return 0;
+        return SDL_OutOfMemory();
     }
-    SDL_memset(this->hidden->mixbuf, this->spec.silence, this->spec.size);
+    SDL_memset(this->hidden->mixbuf, this->spec.silence, this->hidden->mixlen);
 
     /* Switch to blocking mode for playback */
     ALSA_snd_pcm_nonblock(pcm_handle, 0);
 
     /* We're ready to rock and roll. :-) */
-    return 1;
+    return 0;
 }
 
 static void
