@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2011 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2016 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,7 +18,7 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "SDL_config.h"
+#include "../../SDL_internal.h"
 
 #ifndef _SDL_esdaudio_h
 #define _SDL_esdaudio_h
@@ -26,7 +26,7 @@
 #include "../SDL_sysaudio.h"
 
 /* Hidden "this" pointer for the audio functions */
-#define _THIS	SDL_AudioDevice *this
+#define _THIS   SDL_AudioDevice *this
 
 struct SDL_PrivateAudioData
 {
@@ -44,7 +44,7 @@ struct SDL_PrivateAudioData
     float frame_ticks;
     float next_frame;
 };
-#define FUDGE_TICKS	10      /* The scheduler overhead ticks per frame */
+#define FUDGE_TICKS 10      /* The scheduler overhead ticks per frame */
 
 #endif /* _SDL_esdaudio_h */
 /* vi: set ts=4 sw=4 expandtab: */
