@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2011 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2016 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -19,7 +19,16 @@
   3. This notice may not be removed or altered from any source distribution.
 */
 
-#include "SDL_config.h"
+/*
+ * !!! FIXME: streamline this a little by removing all the
+ * !!! FIXME:  if (capture) {} else {} sections that are identical
+ * !!! FIXME:  except for one flag.
+ */
+
+/* !!! FIXME: can this target support hotplugging? */
+/* !!! FIXME: ...does SDL2 even support QNX? */
+
+#include "../../SDL_internal.h"
 
 #if SDL_AUDIO_DRIVER_QSA
 
@@ -83,10 +92,10 @@ uint32_t qsa_playback_devices;
 QSA_Device qsa_capture_device[QSA_MAX_DEVICES];
 uint32_t qsa_capture_devices;
 
-static inline void
+static SDL_INLINE int
 QSA_SetError(const char *fn, int status)
 {
-    SDL_SetError("QSA: %s() failed: %s", fn, snd_strerror(status));
+    return SDL_SetError("QSA: %s() failed: %s", fn, snd_strerror(status));
 }
 
 /* card names check to apply the workarounds */
@@ -183,13 +192,13 @@ QSA_WaitDevice(_THIS)
         switch (selectret) {
         case -1:
             {
-                SDL_SetError("QSA: select() failed: %s\n", strerror(errno));
+                SDL_SetError("QSA: select() failed: %s", strerror(errno));
                 return;
             }
             break;
         case 0:
             {
-                SDL_SetError("QSA: timeout on buffer waiting occured\n");
+                SDL_SetError("QSA: timeout on buffer waiting occured");
                 this->hidden->timeout_on_wait = 1;
                 return;
             }
@@ -237,7 +246,7 @@ QSA_PlayDevice(_THIS)
             /* the audio device driver */
             if ((errno == EAGAIN) && (written == 0)) {
                 if (this->hidden->timeout_on_wait != 0) {
-                    SDL_SetError("QSA: buffer playback timeout\n");
+                    SDL_SetError("QSA: buffer playback timeout");
                     return;
                 }
             }
@@ -300,7 +309,7 @@ QSA_PlayDevice(_THIS)
 
     /* If we couldn't write, assume fatal error for now */
     if (towrite != 0) {
-        this->enabled = 0;
+        SDL_OpenedAudioDeviceDisconnected(this);
     }
 }
 
@@ -328,10 +337,8 @@ QSA_CloseDevice(_THIS)
             this->hidden->audio_handle = NULL;
         }
 
-        if (this->hidden->pcm_buf != NULL) {
-            SDL_FreeAudioMem(this->hidden->pcm_buf);
-            this->hidden->pcm_buf = NULL;
-        }
+        SDL_FreeAudioMem(this->hidden->pcm_buf);
+        this->hidden->pcm_buf = NULL;
 
         SDL_free(this->hidden);
         this->hidden = NULL;
@@ -339,8 +346,9 @@ QSA_CloseDevice(_THIS)
 }
 
 static int
-QSA_OpenDevice(_THIS, const char *devname, int iscapture)
+QSA_OpenDevice(_THIS, void *handle, const char *devname, int iscapture)
 {
+    const QSA_Device *device = (const QSA_Device *) handle;
     int status = 0;
     int format = 0;
     SDL_AudioFormat test_format = 0;
@@ -355,8 +363,7 @@ QSA_OpenDevice(_THIS, const char *devname, int iscapture)
                                                     (struct
                                                      SDL_PrivateAudioData)));
     if (this->hidden == NULL) {
-        SDL_OutOfMemory();
-        return 0;
+        return SDL_OutOfMemory();
     }
     SDL_memset(this->hidden, 0, sizeof(struct SDL_PrivateAudioData));
 
@@ -366,90 +373,26 @@ QSA_OpenDevice(_THIS, const char *devname, int iscapture)
     /* Initialize channel direction: capture or playback */
     this->hidden->iscapture = iscapture;
 
-    /* Find deviceid and cardid by device name for playback */
-    if ((!this->hidden->iscapture) && (devname != NULL)) {
-        uint32_t device;
-        int32_t status;
-
-        /* Search in the playback devices */
-        device = 0;
-        do {
-            status = SDL_strcmp(qsa_playback_device[device].name, devname);
-            if (status == 0) {
-                /* Found requested device */
-                this->hidden->deviceno = qsa_playback_device[device].deviceno;
-                this->hidden->cardno = qsa_playback_device[device].cardno;
-                break;
-            }
-            device++;
-            if (device >= qsa_playback_devices) {
-                QSA_CloseDevice(this);
-                SDL_SetError("No such playback device");
-                return 0;
-            }
-        } while (1);
-    }
-
-    /* Find deviceid and cardid by device name for capture */
-    if ((this->hidden->iscapture) && (devname != NULL)) {
-        /* Search in the capture devices */
-        uint32_t device;
-        int32_t status;
-
-        /* Searching in the playback devices */
-        device = 0;
-        do {
-            status = SDL_strcmp(qsa_capture_device[device].name, devname);
-            if (status == 0) {
-                /* Found requested device */
-                this->hidden->deviceno = qsa_capture_device[device].deviceno;
-                this->hidden->cardno = qsa_capture_device[device].cardno;
-                break;
-            }
-            device++;
-            if (device >= qsa_capture_devices) {
-                QSA_CloseDevice(this);
-                SDL_SetError("No such capture device");
-                return 0;
-            }
-        } while (1);
-    }
-
-    /* Check if SDL requested default audio device */
-    if (devname == NULL) {
-        /* Open system default audio device */
-        if (!this->hidden->iscapture) {
-            status = snd_pcm_open_preferred(&this->hidden->audio_handle,
-                                            &this->hidden->cardno,
-                                            &this->hidden->deviceno,
-                                            SND_PCM_OPEN_PLAYBACK);
-        } else {
-            status = snd_pcm_open_preferred(&this->hidden->audio_handle,
-                                            &this->hidden->cardno,
-                                            &this->hidden->deviceno,
-                                            SND_PCM_OPEN_CAPTURE);
-        }
-    } else {
+    if (device != NULL) {
         /* Open requested audio device */
-        if (!this->hidden->iscapture) {
-            status =
-                snd_pcm_open(&this->hidden->audio_handle,
-                             this->hidden->cardno, this->hidden->deviceno,
-                             SND_PCM_OPEN_PLAYBACK);
-        } else {
-            status =
-                snd_pcm_open(&this->hidden->audio_handle,
-                             this->hidden->cardno, this->hidden->deviceno,
-                             SND_PCM_OPEN_CAPTURE);
-        }
+        this->hidden->deviceno = device->deviceno;
+        this->hidden->cardno = device->cardno;
+        status = snd_pcm_open(&this->hidden->audio_handle,
+                              device->cardno, device->deviceno,
+                              iscapture ? SND_PCM_OPEN_PLAYBACK : SND_PCM_OPEN_CAPTURE);
+    } else {
+        /* Open system default audio device */
+        status = snd_pcm_open_preferred(&this->hidden->audio_handle,
+                                        &this->hidden->cardno,
+                                        &this->hidden->deviceno,
+                                        iscapture ? SND_PCM_OPEN_PLAYBACK : SND_PCM_OPEN_CAPTURE);
     }
 
     /* Check if requested device is opened */
     if (status < 0) {
         this->hidden->audio_handle = NULL;
         QSA_CloseDevice(this);
-        QSA_SetError("snd_pcm_open", status);
-        return 0;
+        return QSA_SetError("snd_pcm_open", status);
     }
 
     if (!QSA_CheckBuggyCards(this, QSA_MMAP_WORKAROUND)) {
@@ -459,8 +402,7 @@ QSA_OpenDevice(_THIS, const char *devname, int iscapture)
                                        PLUGIN_DISABLE_MMAP);
         if (status < 0) {
             QSA_CloseDevice(this);
-            QSA_SetError("snd_pcm_plugin_set_disable", status);
-            return 0;
+            return QSA_SetError("snd_pcm_plugin_set_disable", status);
         }
     }
 
@@ -546,8 +488,7 @@ QSA_OpenDevice(_THIS, const char *devname, int iscapture)
     /* assumes test_format not 0 on success */
     if (test_format == 0) {
         QSA_CloseDevice(this);
-        SDL_SetError("QSA: Couldn't find any hardware audio formats");
-        return 0;
+        return SDL_SetError("QSA: Couldn't find any hardware audio formats");
     }
 
     this->spec.format = test_format;
@@ -565,12 +506,11 @@ QSA_OpenDevice(_THIS, const char *devname, int iscapture)
     status = snd_pcm_plugin_params(this->hidden->audio_handle, &cparams);
     if (status < 0) {
         QSA_CloseDevice(this);
-        QSA_SetError("snd_pcm_channel_params", status);
-        return 0;
+        return QSA_SetError("snd_pcm_channel_params", status);
     }
 
     /* Make sure channel is setup right one last time */
-    SDL_memset(&csetup, '\0', sizeof(csetup));
+    SDL_memset(&csetup, 0, sizeof(csetup));
     if (!this->hidden->iscapture) {
         csetup.channel = SND_PCM_CHANNEL_PLAYBACK;
     } else {
@@ -580,8 +520,7 @@ QSA_OpenDevice(_THIS, const char *devname, int iscapture)
     /* Setup an audio channel */
     if (snd_pcm_plugin_setup(this->hidden->audio_handle, &csetup) < 0) {
         QSA_CloseDevice(this);
-        SDL_SetError("QSA: Unable to setup channel\n");
-        return 0;
+        return SDL_SetError("QSA: Unable to setup channel");
     }
 
     /* Calculate the final parameters for this audio specification */
@@ -604,8 +543,7 @@ QSA_OpenDevice(_THIS, const char *devname, int iscapture)
         (Uint8 *) SDL_AllocAudioMem(this->hidden->pcm_len);
     if (this->hidden->pcm_buf == NULL) {
         QSA_CloseDevice(this);
-        SDL_OutOfMemory();
-        return 0;
+        return SDL_OutOfMemory();
     }
     SDL_memset(this->hidden->pcm_buf, this->spec.silence,
                this->hidden->pcm_len);
@@ -623,8 +561,7 @@ QSA_OpenDevice(_THIS, const char *devname, int iscapture)
 
     if (this->hidden->audio_fd < 0) {
         QSA_CloseDevice(this);
-        QSA_SetError("snd_pcm_file_descriptor", status);
-        return 0;
+        return QSA_SetError("snd_pcm_file_descriptor", status);
     }
 
     /* Prepare an audio channel */
@@ -642,16 +579,15 @@ QSA_OpenDevice(_THIS, const char *devname, int iscapture)
 
     if (status < 0) {
         QSA_CloseDevice(this);
-        QSA_SetError("snd_pcm_plugin_prepare", status);
-        return 0;
+        return QSA_SetError("snd_pcm_plugin_prepare", status);
     }
 
     /* We're really ready to rock and roll. :-) */
-    return 1;
+    return 0;
 }
 
 static void
-QSA_DetectDevices(int iscapture, SDL_AddAudioDevice addfn)
+QSA_DetectDevices(void)
 {
     uint32_t it;
     uint32_t cards;
@@ -669,8 +605,9 @@ QSA_DetectDevices(int iscapture, SDL_AddAudioDevice addfn)
         return;
     }
 
+    /* !!! FIXME: code duplication */
     /* Find requested devices by type */
-    if (!iscapture) {
+    {  /* output devices */
         /* Playback devices enumeration requested */
         for (it = 0; it < cards; it++) {
             devices = 0;
@@ -701,7 +638,7 @@ QSA_DetectDevices(int iscapture, SDL_AddAudioDevice addfn)
                             devices;
                         status = snd_pcm_close(handle);
                         if (status == EOK) {
-                            addfn(qsa_playback_device[qsa_playback_devices].name);
+                            SDL_AddAudioDevice(SDL_FALSE, qsa_playback_device[qsa_playback_devices].name, &qsa_playback_device[qsa_playback_devices]);
                             qsa_playback_devices++;
                         }
                     } else {
@@ -726,7 +663,9 @@ QSA_DetectDevices(int iscapture, SDL_AddAudioDevice addfn)
                 break;
             }
         }
-    } else {
+    }
+
+    {  /* capture devices */
         /* Capture devices enumeration requested */
         for (it = 0; it < cards; it++) {
             devices = 0;
@@ -757,7 +696,7 @@ QSA_DetectDevices(int iscapture, SDL_AddAudioDevice addfn)
                             devices;
                         status = snd_pcm_close(handle);
                         if (status == EOK) {
-                            addfn(qsa_capture_device[qsa_capture_devices].name);
+                            SDL_AddAudioDevice(SDL_TRUE, qsa_capture_device[qsa_capture_devices].name, &qsa_capture_device[qsa_capture_devices]);
                             qsa_capture_devices++;
                         }
                     } else {
