@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2011 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2016 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,7 +18,7 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "SDL_config.h"
+#include "../../SDL_internal.h"
 
 #if SDL_VIDEO_DRIVER_PANDORA
 
@@ -38,7 +38,7 @@
 /* WIZ declarations */
 #include "GLES/gl.h"
 #ifdef WIZ_GLES_LITE
-static NativeWindowType hNativeWnd = 0; // A handle to the window we will create.
+static NativeWindowType hNativeWnd = 0; /* A handle to the window we will create. */
 #endif
 
 static SDL_bool PND_initialized = SDL_FALSE;
@@ -131,6 +131,8 @@ PND_create()
     device->GL_DeleteContext = PND_gl_deletecontext;
     device->PumpEvents = PND_PumpEvents;
 
+    /* !!! FIXME: implement SetWindowBordered */
+
     return device;
 }
 
@@ -209,8 +211,7 @@ PND_createwindow(_THIS, SDL_Window * window)
     /* Allocate window internal data */
     wdata = (SDL_WindowData *) SDL_calloc(1, sizeof(SDL_WindowData));
     if (wdata == NULL) {
-        SDL_OutOfMemory();
-        return -1;
+        return SDL_OutOfMemory();
     }
 
     /* Setup driver data for this window */
@@ -228,14 +229,12 @@ PND_createwindow(_THIS, SDL_Window * window)
         if (phdata->egl_display == EGL_NO_DISPLAY) {
             phdata->egl_display = eglGetDisplay((NativeDisplayType) 0);
             if (phdata->egl_display == EGL_NO_DISPLAY) {
-                SDL_SetError("PND: Can't get connection to OpenGL ES");
-                return -1;
+                return SDL_SetError("PND: Can't get connection to OpenGL ES");
             }
 
             initstatus = eglInitialize(phdata->egl_display, NULL, NULL);
             if (initstatus != EGL_TRUE) {
-                SDL_SetError("PND: Can't init OpenGL ES library");
-                return -1;
+                return SDL_SetError("PND: Can't init OpenGL ES library");
             }
         }
 
@@ -295,8 +294,6 @@ PND_restorewindow(_THIS, SDL_Window * window)
 void
 PND_setwindowgrab(_THIS, SDL_Window * window)
 {
-    SDL_VideoData *phdata = (SDL_VideoData *) _this->driverdata;
-    eglTerminate(phdata->egl_display);
 }
 void
 PND_destroywindow(_THIS, SDL_Window * window)
@@ -344,7 +341,7 @@ PND_gl_loadlibrary(_THIS, const char *path)
         /* Already linked with GF library which provides egl* subset of  */
         /* functions, use Common profile of OpenGL ES library by default */
 #ifdef WIZ_GLES_LITE
-	path = "/lib/libopengles_lite.so";
+    path = "/lib/libopengles_lite.so";
 #else
         path = "/usr/lib/libGLES_CM.so";
 #endif
@@ -354,8 +351,7 @@ PND_gl_loadlibrary(_THIS, const char *path)
     _this->gl_config.dll_handle = SDL_LoadObject(path);
     if (!_this->gl_config.dll_handle) {
         /* Failed to load new GL ES library */
-        SDL_SetError("PND: Failed to locate OpenGL ES library");
-        return -1;
+        return SDL_SetError("PND: Failed to locate OpenGL ES library");
     }
 
     /* Store OpenGL ES library path and name */
@@ -630,21 +626,21 @@ PND_gl_createcontext(_THIS, SDL_Window * window)
 
 #ifdef WIZ_GLES_LITE
     if( !hNativeWnd ) {
-	hNativeWnd = (NativeWindowType)malloc(16*1024);
+    hNativeWnd = (NativeWindowType)malloc(16*1024);
 
-	if(!hNativeWnd)
-	    printf( "Error : Wiz framebuffer allocatation failed\n" ); 
-	else
-	    printf( "SDL13: Wiz framebuffer allocated: %X\n", hNativeWnd );
+    if(!hNativeWnd)
+        printf( "Error : Wiz framebuffer allocatation failed\n" );
+    else
+        printf( "SDL13: Wiz framebuffer allocated: %X\n", hNativeWnd );
     }
     else {
-	printf( "SDL13: Wiz framebuffer already allocated: %X\n", hNativeWnd );
+    printf( "SDL13: Wiz framebuffer already allocated: %X\n", hNativeWnd );
     }
 
     wdata->gles_surface =
-	eglCreateWindowSurface(phdata->egl_display, 
-			       wdata->gles_configs[wdata->gles_config],
-			       hNativeWnd, NULL );
+    eglCreateWindowSurface(phdata->egl_display,
+                   wdata->gles_configs[wdata->gles_config],
+                   hNativeWnd, NULL );
 #else
     wdata->gles_surface =
         eglCreateWindowSurface(phdata->egl_display,
@@ -724,8 +720,7 @@ PND_gl_makecurrent(_THIS, SDL_Window * window, SDL_GLContext context)
     EGLBoolean status;
 
     if (phdata->egl_initialized != SDL_TRUE) {
-        SDL_SetError("PND: GF initialization failed, no OpenGL ES support");
-        return -1;
+        return SDL_SetError("PND: GF initialization failed, no OpenGL ES support");
     }
 
     if ((window == NULL) && (context == NULL)) {
@@ -734,33 +729,28 @@ PND_gl_makecurrent(_THIS, SDL_Window * window, SDL_GLContext context)
                            EGL_NO_SURFACE, EGL_NO_CONTEXT);
         if (status != EGL_TRUE) {
             /* Failed to set current GL ES context */
-            SDL_SetError("PND: Can't set OpenGL ES context");
-            return -1;
+            return SDL_SetError("PND: Can't set OpenGL ES context");
         }
     } else {
         wdata = (SDL_WindowData *) window->driverdata;
         if (wdata->gles_surface == EGL_NO_SURFACE) {
-            SDL_SetError
+            return SDL_SetError
                 ("PND: OpenGL ES surface is not initialized for this window");
-            return -1;
         }
         if (wdata->gles_context == EGL_NO_CONTEXT) {
-            SDL_SetError
+            return SDL_SetError
                 ("PND: OpenGL ES context is not initialized for this window");
-            return -1;
         }
         if (wdata->gles_context != context) {
-            SDL_SetError
+            return SDL_SetError
                 ("PND: OpenGL ES context is not belong to this window");
-            return -1;
         }
         status =
             eglMakeCurrent(phdata->egl_display, wdata->gles_surface,
                            wdata->gles_surface, wdata->gles_context);
         if (status != EGL_TRUE) {
             /* Failed to set current GL ES context */
-            SDL_SetError("PND: Can't set OpenGL ES context");
-            return -1;
+            return SDL_SetError("PND: Can't set OpenGL ES context");
         }
     }
     return 0;
@@ -773,8 +763,7 @@ PND_gl_setswapinterval(_THIS, int interval)
     EGLBoolean status;
 
     if (phdata->egl_initialized != SDL_TRUE) {
-        SDL_SetError("PND: EGL initialization failed, no OpenGL ES support");
-        return -1;
+        return SDL_SetError("PND: EGL initialization failed, no OpenGL ES support");
     }
 
     /* Check if OpenGL ES connection has been initialized */
@@ -789,22 +778,13 @@ PND_gl_setswapinterval(_THIS, int interval)
     }
 
     /* Failed to set swap interval */
-    SDL_SetError("PND: Cannot set swap interval");
-    return -1;
+    return SDL_SetError("PND: Cannot set swap interval");
 }
 
 int
 PND_gl_getswapinterval(_THIS)
 {
-    SDL_VideoData *phdata = (SDL_VideoData *) _this->driverdata;
-
-    if (phdata->egl_initialized != SDL_TRUE) {
-        SDL_SetError("PND: GLES initialization failed, no OpenGL ES support");
-        return -1;
-    }
-
-    /* Return default swap interval value */
-    return phdata->swapinterval;
+    return ((SDL_VideoData *) _this->driverdata)->swapinterval;
 }
 
 void
@@ -856,9 +836,9 @@ PND_gl_deletecontext(_THIS, SDL_GLContext context)
 #ifdef WIZ_GLES_LITE
     if( hNativeWnd != 0 )
     {
-	  free(hNativeWnd);
-	  hNativeWnd = 0;
-	  printf( "SDL13: Wiz framebuffer released\n" );
+      free(hNativeWnd);
+      hNativeWnd = 0;
+      printf( "SDL13: Wiz framebuffer released\n" );
     }
 #endif
 
