@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2011 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2016 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,26 +18,11 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "SDL_config.h"
+#include "../../SDL_internal.h"
 
 #ifndef _SDL_x11mouse_h
 #define _SDL_x11mouse_h
 
-#if SDL_VIDEO_DRIVER_X11_XINPUT
-typedef struct X11_MouseData
-{
-    Display *display;
-    XDevice *device;
-    int motion;
-    int button_pressed;
-    int button_released;
-    int proximity_in;
-    int proximity_out;
-    int num_xevents;
-    XEventClass xevents[5];
-} X11_MouseData;
-#endif
-
 extern void X11_InitMouse(_THIS);
 extern void X11_QuitMouse(_THIS);
 
